/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2023 Engineering Design Team, Inc.
 * @brief IDs of EDT "PCI Configurable DMA" (PCD) class devices.
 */

#ifndef EDT_PCI_DEVICES_PCD_H
#define EDT_PCI_DEVICES_PCD_H

#ifdef __cplusplus
extern "C" {
#endif

/* X-macro of EDT PCI devices and their properties
 *
 * X(_defname, _pid, _dma_ch, _name, _desc)
 *
 * - PCI PID define name
 * - PCI PID value
 * - Number of DMA channels
 * - Name (Short, lowercase, underscores, no spaces. Useful as filenames.)
 * - Description
 */
#define EDT_PCI_DEVICES_LISTX(X)                                                        \
  X(PE4CDA_ID, 0x0047U, 4, "pe4g1_cda", "PCIe4 CDa")                                    \
  X(PE4CDA16_ID, 0x004bU, 16, "pe4g1_cda16", "PCIe4 CDa 16")                            \
  X(PE8G3S5_ID, 0x00a0U, 16, "pe8g3_s5", "PCIe8 S5 (10G/40G)")                          \
  X(PE8G3A5_ID, 0x00abU, 16, "pe8g3_a5", "PCIe8 A5 10G")                                \
  X(PE8LX1_ID, 0x0096U, 1, "pe8g1_lx1", "PCIe8 LX/FX/SX 1ch")                           \
  X(PE8LX16_ID, 0x0097U, 16, "pe8g1_lx16", "PCIe8 LX/FX/SX 16ch")                       \
  X(PE8LX16_LS_ID, 0x0098U, 16, "pe8g1_lx16_ls", "PCIe8 LX/FX/SX 16ch (low speed)")     \
  X(PE8LX32_ID, 0x009cU, 32, "pe8g1_lx32", "PCIe8 LX/FX/SX 32ch")                       \
  X(PE4BL_RADIO_ID, 0x00a4U, 16, "pe4g2bl_lcr_radio", "LCR Radio Blade")                \
  X(PE1BL_TIMING_ID, 0x00a5U, 1, "pe1g2bl_lcr_timing", "LCR Timing Blade")              \
  X(PE8BL_NIC_ID, 0x00a6U, 16, "pe8g2bl_lcr_nic", "LCR NIC Blade (10G/2x10G)")          \
  X(LCRBOOT_ID, 0x00a7U, 1, "pe4g2bl_lcr_boot", "LCR Boot")                             \
  X(PE4BL_RXLFRADIO_ID, 0x00adU, 16, "pe4g2bl_lcr_radio_rxlf", "LCR RX/LF Radio Blade") \
  X(PE4BL_TXLFRADIO_ID, 0x00afU, 16, "pe4g2bl_lcr_radio_txlf", "LCR TX/LF Radio Blade") \
  X(PE8G2CML_ID, 0x00b0U, 16, "pe8g2_cml_ecl", "PCIe8 CML-ECL")                         \
  X(PE1BL_WBADC_ID, 0x00b1U, 1, "pe1g2bl_wbadc", "WBX Analog Wideband Radio Blade")     \
  X(PE8BL_WBDSP_ID, 0x00b2U, 16, "pe8g2bl_wbdsp", "WBX DSP Blade")                      \
  X(PE16G3_OCTEON3_ID, 0x00b3U, 4, "pe16g3_octeon3", "PCIe 100G NIC")                   \
  X(PE8BL_40G_ID, 0x00b4U, 16, "pe8g2bl_nic10g", "WBX 40G NIC Blade")                   \
  X(PE8G2_RFX_RADIO_ID, 0x00b5U, 16, "pe8g2_rfx_radio", "PCIe8 RFx SDR")

/*-----------------------------------------------------------------------------
 * Device ID aliases
 *---------------------------------------------------------------------------*/

#define PE84P_ID PE8G3S5_ID

/* LCR aliases */
#define PE8BL_2X10G_ID   PE8BL_NIC_ID
#define PE8BL_10GNI_ID   PE8BL_NIC_ID
#define PE8BL_10GNIC_ID  PE8BL_NIC_ID
#define PE4BL_LFRADIO_ID PE4BL_RXLFRADIO_ID /* old (compat) */
#define PE4BL_TXRADIO_ID PE4BL_TXLFRADIO_ID /* old (compat) */

/*-----------------------------------------------------------------------------
 * Mezzanine ID values
 *---------------------------------------------------------------------------*/

/* X-macro of EDT Mezzanine devices and their properties
 *
 * X(_defname, _id, _name, _desc)
 *
 * - MEZZ ID define name
 * - MEZZ ID value
 * - Name (Short, lowercase, underscores, no spaces. Useful as filenames.)
 * - Description
 */
#define EDT_MEZZANINE_DEVICES_LISTX(X)                                       \
  X(MEZZ_UNKN_EXTBDID, 0x2U, "mezz_unkn_ext", "UNKNOWN Extended")            \
  X(MEZZ_SSE, 0x4U, "mezz_sse", "SSE")                                       \
  X(MEZZ_COMBO2, 0x7U, "mezz_combo2", "COMBO II LVDS")                       \
  X(MEZZ_ECL, 0x8U, "mezz_ecl", "ECL")                                       \
  X(MEZZ_SRXL, 0xaU, "mezz_srxl", "SRXL")                                    \
  X(MEZZ_COMBO3_RS422, 0xbU, "mezz_combo3_rs422", "COMBO III RS422")         \
  X(MEZZ_COMBO3_LVDS, 0xcU, "mezz_combo3_lvds", "COMBO III LVDS")            \
  X(MEZZ_COMBO3_ECL, 0xdU, "mezz_combo3_ecl", "COMBO III ECL")               \
  X(MEZZ_COMBO2_RS422, 0xeU, "mezz_combo2_rs422", "COMBO II RS422")          \
  X(MEZZ_COMBO, 0xfU, "mezz_combo", "COMBO")                                 \
  X(MEZZ_3X3G, 0x12U, "mezz_3x3g", "3x3G")                                   \
  X(MEZZ_SRXL2, 0x14U, "mezz_srxl2", "SRXL2")                                \
  X(MEZZ_SRXL2_IDM_LBM, 0x18U, "mezz_srxl2", "SRXL2 (IF direct, L-band)")    \
  X(MEZZ_SRXL2_IDM_IDM, 0x19U, "mezz_srxl2", "SRXL2 (IF direct, IF direct)") \
  X(MEZZ_SRXL2_IMM_IMM, 0x1aU, "mezz_srxl2", "SRXL2 (IF mixer, IF mixer)")   \
  X(MEZZ_SRXL2_IMM_LBM, 0x1bU, "mezz_srxl2", "SRXL2 (IF mixer, L-band)")     \
  X(MEZZ_SRXL2_IDM_IMM, 0x1cU, "mezz_srxl2", "SRXL2 (IF direct, IF mixer)")  \
  X(MEZZ_DRX16, 0x1dU, "mezz_drx16", "DRX16")                                \
  X(MEZZ_THREEP, 0x1fU, "mezz_3p", "3P")                                     \
  X(MEZZ_FAN, 0x20U, "mezz_fan", "Fan")                                      \
  X(MEZZ_V4ACL, 0x21U, "mezz_v4acl", "V4ACL")

#define MEZZ_LAST MEZZ_V4ACL

#define MEZZ_ERR_NO_BITFILE    0xcc
#define MEZZ_ERR_NO_REGISTER   0xff
#define MEZZ_ERR_BAD_BITSTREAM 0xffffffffU /* problem with extbdid lookup */

/* Enum of EDT Mezzanine device IDs */
enum EDT_MEZZ_ID {
  MEZZ_ID_UNKNOWN = 0xffffU,
#define EXPAND_AS_DEFNAME_ENUM(_defname, _id, _name, _desc) _defname = _id,

  EDT_MEZZANINE_DEVICES_LISTX(EXPAND_AS_DEFNAME_ENUM)

#undef EXPAND_AS_DEFNAME_ENUM
};

/*-----------------------------------------------------------------------------
 * "ID_IS" macros
 *---------------------------------------------------------------------------*/

#define EDT_DEVID_IS_PCD(id)                                                                 \
  ((id == PE4CDA_ID) || (id == PE4CDA16_ID) || (id == PE8LX1_ID) || (id == PE8LX16_LS_ID) || \
   (id == PE8LX16_ID) || (id == PE8LX32_ID) || (id == PE8G3S5_ID) || (id == PE8G3A5_ID) ||   \
   (id == PE8G2CML_ID) || (id == PE4BL_RADIO_ID) || (id == PE8G2_RFX_RADIO_ID) ||            \
   (id == PE4BL_RXLFRADIO_ID) || (id == PE4BL_TXLFRADIO_ID) || (id == PE8BL_NIC_ID) ||       \
   (id == PE1BL_TIMING_ID) || (id == LCRBOOT_ID) || (id == PE8BL_WBDSP_ID) ||                \
   (id == PE1BL_WBADC_ID) || (id == PE16G3_OCTEON3_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_IS_1LANE(id) ((id == PE1BL_TIMING_ID) || (id == PE1BL_WBADC_ID))

#define EDT_DEVID_IS_4LANE(id)                                           \
  ((id == PE4CDA_ID) || (id == PE4CDA16_ID) || (id == PE4BL_RADIO_ID) || \
   (id == PE4BL_RXLFRADIO_ID) || (id == PE4BL_TXLFRADIO_ID) || (id == PE16G3_OCTEON3_ID))

#define EDT_DEVID_IS_8LANE(id)                                                                 \
  ((id == PE8LX1_ID) || (id == PE8LX16_ID) || (id == PE8LX16_LS_ID) || (id == PE8BL_NIC_ID) || \
   (id == PE8LX32_ID) || (id == PE8G3S5_ID) || (id == PE8G3A5_ID) || (id == PE8G2CML_ID) ||    \
   (id == PE8BL_WBDSP_ID) || (id == PE8BL_40G_ID) || (id == PE8G2_RFX_RADIO_ID))

#define EDT_DEVID_IS_LX(id) \
  ((id == PE8LX1_ID) || (id == PE8LX16_ID) || (id == PE8LX16_LS_ID) || (id == PE8LX32_ID))

#define EDT_DEVID_STORES_MACADDRS(id)                                                             \
  ((EDT_DEVID_HAS_MEZZ(id)) || (id == PE8G3S5_ID) || (id == PE8G3A5_ID) || (id == PE8G2CML_ID) || \
   (id == LCRBOOT_ID) || (id == PE8BL_NIC_ID) || (id == PE8BL_WBDSP_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_IS_LCRBLADE(id)                                                        \
  ((id == PE4BL_RADIO_ID) || (id == PE4BL_RXLFRADIO_ID) || (id == PE4BL_TXLFRADIO_ID) || \
   (id == PE1BL_TIMING_ID) || (id == PE8BL_NIC_ID) || (id == PE8BL_WBDSP_ID) ||          \
   (id == PE1BL_WBADC_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_IS_RADIOBLADE(id)                                                      \
  ((id == PE4BL_RADIO_ID) || (id == PE8G2_RFX_RADIO_ID) || (id == PE4BL_RXLFRADIO_ID) || \
   (id == PE4BL_TXLFRADIO_ID))

/* ADD any devices that don't have a separate interface FPGA here */
#define EDT_DEVID_HAS_COMBINED_FPGA(id)                                                    \
  ((id == PE4CDA_ID) || (id == PE4CDA16_ID) || (id == PE8G3S5_ID) || (id == PE8G3A5_ID) || \
   (id == PE4BL_RADIO_ID) || (id == PE8G2_RFX_RADIO_ID) || (id == PE4BL_RXLFRADIO_ID) ||   \
   (id == PE4BL_TXLFRADIO_ID) || (id == PE1BL_TIMING_ID) || (id == PE8BL_NIC_ID) ||        \
   (id == LCRBOOT_ID) || (id == PE8BL_WBDSP_ID) || (id == PE1BL_WBADC_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_IS_MULTICHAN(id)                                                           \
  ((id == PE4CDA_ID) || (id == PE4CDA16_ID) || (id == PE8LX16_ID) || (id == PE8LX32_ID) ||   \
   (id == PE8BL_NIC_ID) || (id == PE8LX32_ID) || (id == PE8G3S5_ID) || (id == PE8G3A5_ID) || \
   (id == PE8G2CML_ID) || (id == PE4BL_RADIO_ID) || (id == PE8G2_RFX_RADIO_ID) ||            \
   (id == PE4BL_RXLFRADIO_ID) || (id == PE4BL_TXLFRADIO_ID) || (id == PE8LX16_LS_ID) ||      \
   (id == PE8BL_WBDSP_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_HAS_16BIT_PROM(id)                                                       \
  ((id == PE8G3S5_ID) || (id == PE8G3A5_ID) || (id == PE4CDA_ID) || (id == PE4CDA16_ID) || \
   (id == PE8G2CML_ID))

#define EDT_DEVID_IS_MICRON_PROM(id)                                                               \
  ((id == PE4BL_RADIO_ID) || (id == PE8G2_RFX_RADIO_ID) || (id == PE4BL_RXLFRADIO_ID) ||           \
   (id == PE4BL_TXLFRADIO_ID) || (id == PE1BL_TIMING_ID) || (id == PE8BL_NIC_ID) ||                \
   (id == PE8G3A5_ID) || (id == LCRBOOT_ID) || (id == PE8BL_WBDSP_ID) || (id == PE1BL_WBADC_ID) || \
   (id == PE16G3_OCTEON3_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_HAS_PCD_DIR_BIT(id) (id == PE4CDA_ID)

#define EDT_DEVID_IS_1OR4CHANNEL(id) (id == PE4CDA_ID)

#define EDT_DEVID_HAS_MEZZ(id) (EDT_DEVID_IS_LX(id))

#define EDT_DEVID_HAS_CHANREG(id) EDT_DEVID_HAS_MEZZ(id)

#define EDT_DEVID_HAS_SUBUNIT(id) ((id == PE8BL_WBDSP_ID) || (id == PE8BL_40G_ID))

#define EDT_DEVID_PCILOAD_INFO_NA(id) (0 == 1) /* false */

#ifdef __cplusplus
} /* extern C */
#endif

#endif /* EDT_PCI_DEVICES_PCD_H */
